#pragma once

#include <vector>

class Pattern;

// solving Least Squares problem
// y = a1*x1 + a2*x2 + a3*x3 + .... + an*xn + b
// SUM(y - e)^2 --> min

class Solver
{
public:
	explicit Solver(size_t patternSize, bool alwaysSolve = true);

public:
	void FeedPatterns(const std::vector<Pattern>& patterns);
	void FeedPattern(const Pattern& pattern);

	const std::vector<double>& Solve();
	const std::vector<double>& Solution() const;

private:
	void Substract(size_t whatIdx, size_t fromIdx, double coef);
	void Divide(size_t whatIdx, double coef);

	void Print();

	void InitMatrix();

	size_t FindZeroLine() const;

private:
	size_t m_PatternSize;
	std::vector<std::vector<double> > m_Matrix;
	std::vector<double> m_Solution;
	bool m_SolveAlways;

	std::vector<bool> m_ZeroMarkers;
};

// solving Least Squares problem
// y = a1*x1 + a2*x2 + a3*x3 + .... + an*xn
// SUM(y - e)^2 --> min

class NoThresholdSolver
{
public:
	explicit NoThresholdSolver(size_t patternSize, bool alwaysSolve = true);

public:
	void FeedPatterns(const std::vector<Pattern>& patterns);
	void FeedPattern(const Pattern& pattern);

	const std::vector<double>& Solve();
	const std::vector<double>& Solution() const;


private:
	void Substract(size_t whatIdx, size_t fromIdx, double coef);
	void Divide(size_t whatIdx, double coef);

	void Print();

	void InitMatrix();

	size_t FindZeroLine() const;

private:
	size_t m_PatternSize;
	std::vector<std::vector<double> > m_Matrix;
	std::vector<double> m_Solution;
	bool m_SolveAlways;
};